#! perl

# Copyright (C) 2007-2008, The Perl Foundation.
# $Id: /mirror/trunk/languages/eclectus/t/binary_primitives.pl 29457 2008-07-14T11:41:28.412963Z bernhard  $

use strict;
use warnings;

do 'eclectus/test-wrapper.pl';

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 100
# End:
# vim: expandtab shiftwidth=4:
