; $Id: /mirror/trunk/languages/eclectus/t/vectors.t 26881 2008-04-09T17:11:30.653722Z bernhard  $

(load "tests-driver.scm") ; this should come first

(add-tests-with-string-output "vectors"
  ('#()                   => "#0()\n")                    
)

(load "compiler.scm")
(test-all)
