#!perl
# Copyright (C) 2008, The Perl Foundation.
# $Id: /mirror/trunk/compilers/pirc/t/subflags.t 32833 2008-11-18T08:43:22.778135Z kjs  $


use lib "../../lib";
use Parrot::Test tests => 1;

pirc_2_pasm_is(<<'CODE', <<'OUTPUT', "test for :main");
.sub main :main
.end
CODE
.namespace []

.pcc_sub :main main:
    get_params
    end
OUTPUT


# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 100
# End:
# vim: expandtab shiftwidth=4:
