/*
 * $Id: /mirror/trunk/compilers/bcg/src/bcg_emitter.h 21219 2007-09-12T17:33:53.357739Z paultcochrane  $
 * Copyright (C) 2006-2007, The Perl Foundation.
 */

#ifndef PARROT_BCG_EMITTER_H_GUARD
#define PARROT_BCG_EMITTER_H_GUARD

#include "bcg.h"

/* TODO Define standard emitter API that can support multiple emitter*/
void emit_pasm(BCG_info * info);

#endif /* PARROT_BCG_EMITTER_H_GUARD */

/*
 * Local variables:
 *   c-file-style: "parrot"
 * End:
 * vim: expandtab shiftwidth=4:
 */
