# $Id: /mirror/trunk/languages/m4/t/basic/001_completely_empty.t 28681 2008-06-22T17:12:49.800522Z jkeenan  $

use strict;
use warnings;
use lib qw( lib ../lib ../../lib m4/lib );

use Parrot::Test tests => 1;

language_output_is( 'm4', <<'CODE', <<'OUT', 'completely empty' );
CODE
OUT
