#! parrot
# Copyright (C) 2006-2008, The Perl Foundation.
# $Id: /mirror/trunk/t/pmc/pointer.t 25826 2008-02-18T00:16:21.490499Z coke  $

=head1 NAME

t/pmc/pointer.t - test the Pointer PMC

=head1 SYNOPSIS

    % prove t/pmc/pointer.t

=head1 DESCRIPTION

Tests the Pointer PMC.

=cut

.sub main :main
    .include 'include/test_more.pir'

    plan(1)

    new P0, 'Pointer'
    ok(1, 'Instantiated .Pointer')
.end

# Local Variables:
#   mode: pir
#   fill-column: 100
# End:
# vim: expandtab shiftwidth=4 ft=pir:
