; $Id: /mirror/trunk/languages/eclectus/t/empty_list.t 27302 2008-05-02T19:30:17.715999Z bernhard  $

(load "tests-driver.scm") ; this should come first

(add-tests-with-string-output "empty_list"
  ('()  => "()\n"))

(load "compiler.scm")
(test-all)
