; $Id: /mirror/trunk/languages/eclectus/t/io.t 27317 2008-05-03T13:26:34.789238Z bernhard  $

(load "tests-driver.scm") ; this should come first

(add-tests-with-string-output "io"
  ((newline)                   => "\n\n")                    
  ((begin (newline)(newline))  => "\n\n\n")                    
)

(load "compiler.scm")
(test-all)
