; $Id: /mirror/trunk/languages/eclectus/t/strings.t 26867 2008-04-08T22:36:28.646747Z bernhard  $

(load "tests-driver.scm") ; this should come first

(add-tests-with-string-output "strings"      
  ("asdf"          => "asdf\n")                    
  ("1"             => "1\n")                    
  ; ("\n"            => "\n\n")                    
  (""              => "\n")                    
)

(load "compiler.scm")
(test-all)
