; $Id: /mirror/trunk/languages/eclectus/t/booleans.t 27198 2008-04-26T17:20:45.657924Z bernhard  $

(load "tests-driver.scm") ; this should come first

(add-tests-with-string-output "booleans"      
  (#t  => "#t\n")                    
  (#f  => "#f\n"))

(load "compiler.scm")
(test-all)
