/*
 * $Id: /mirror/trunk/compilers/pirc/new/pirheredoc.h 32217 2008-10-26T21:55:23.528200Z kjs  $
 * Copyright (C) 2008, The Perl Foundation.
 */


#ifndef PARROT_PIR_PIRHEREDOC_H_GUARD
#define PARROT_PIR_PIRHEREDOC_H_GUARD

#include <stdio.h> /* for FILE */

void process_heredocs(char * const filename, FILE *outputfile);

#endif /* PARROT_PIR_PIRHEREDOC_H_GUARD */

/*
 * Local variables:
 *   c-file-style: "parrot"
 * End:
 * vim: expandtab shiftwidth=4:
 */

