#!perl
# Copyright (C) 2008, The Perl Foundation.
# $Id: /mirror/trunk/compilers/pirc/t/macro.t 32831 2008-11-18T08:34:50.293378Z kjs  $

use lib "../../lib";
use Parrot::Test tests => 1;


pirc_2_pasm_is(<<'CODE', <<'OUTPUT', "simple macro without parameters");
.macro Hi()
    print "hello"
.endm
.sub main
    .Hi()
.end
CODE
 .namespace []
main:
    get_params
    print "hello"
    set_returns
    returncc
OUTPUT

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 100
# End:
# vim: expandtab shiftwidth=4:

