/*
 * $Id: /mirror/trunk/compilers/pirc/src/jsonout.h 21221 2007-09-12T17:41:19.474159Z paultcochrane  $
 * Copyright (C) 2007, The Perl Foundation.
 */

#ifndef PARROT_PIRC_JSONOUT_H_GUARD
#define PARROT_PIRC_JSONOUT_H_GUARD

/* predeclare */
struct pirvtable;

extern struct pirvtable *init_json_vtable(char *outputfile);

#endif /* PARROT_PIRC_JSONOUT_H_GUARD */


/*
 * Local variables:
 *   c-file-style: "parrot"
 * End:
 * vim: expandtab shiftwidth=4:
 */

